/*
** Copyright (C) 6 July 1999 George Saj <gsaj@acm.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <signal.h>
#include <stdlib.h>
#include <errno.h> 
#include <gtk/gtk.h>
#include "menubar.h"
#include "psinfo.h"


static gint selected_row;

void error_dialog1(gchar *text){
     GtkWidget *errdialog, *button, *errmsg;

     errdialog = GTK_WIDGET (create_simple_dialog("Error"));
     gtk_container_border_width (GTK_CONTAINER (errdialog), 10);
     errmsg = gtk_label_new(text);

     gtk_box_pack_start(GTK_BOX(GTK_DIALOG (errdialog)->vbox),
			errmsg, TRUE, TRUE, 0);
     gtk_widget_show(errmsg);
     
     button = gtk_button_new_with_label("OK");
     gtk_signal_connect (GTK_OBJECT(button),"clicked",
			 GTK_SIGNAL_FUNC(close_dialog),
			 errdialog);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (errdialog)->action_area),
			 button, TRUE,TRUE,0);
     gtk_widget_grab_default(button);
     gtk_widget_show(button);
     gtk_widget_show(errdialog);
     gtk_grab_add(errdialog);

}


void select_row_callback(GtkWidget *widget, gint row,
			 gint column, GdkEventButton *event,
			 gpointer data) {
     
     selected_row = row;
}

void send_signal(GtkWidget *clist, gint signal){
     gchar *pid_str;
     int kill_result = 0;

     gtk_clist_get_text(GTK_CLIST(clist),selected_row,1, &pid_str);
   
     kill_result = kill((pid_t)strtod(pid_str,0), signal);
     if (kill_result == -1) {
	  gchar failure[100]; 
	  snprintf(failure, sizeof(failure), "error: %s", strerror(errno));
	  error_dialog1(failure);
     }    
     else
	  RESCAN();
}

void end_proc(GtkWidget *button, GtkWidget *clist) {
     send_signal(clist,SIGHUP);
}

void kill_proc(GtkWidget *button, GtkWidget *clist) {
     send_signal(clist,SIGKILL); 
}

/* Window destruction callback */
gint DestroyWindow(GtkWidget *widget, gpointer *data){
  gtk_main_quit();
  return FALSE;
}


int main( int   argc,
	  char *argv[] )
{
  GtkWidget *window, *clist1, *separator, *vbox1, *frame, *button;
  GtkWidget *menubar, *scrollwindow, *hbox1;

  gtk_init (&argc, &argv);
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW (window), "gtaskman");
  gtk_widget_set_usize(window, 426, 300);
  gtk_container_border_width(GTK_CONTAINER (window), 0);
  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		      GTK_SIGNAL_FUNC (DestroyWindow), NULL);
  
  vbox1 = gtk_vbox_new(FALSE, 0);
  gtk_container_border_width(GTK_CONTAINER(vbox1), 1);

  menubar = get_main_menu(window);
  gtk_box_pack_start(GTK_BOX(vbox1), menubar, FALSE, TRUE, 0);
  

  /* CList, in a scrolled window */
  scrollwindow = gtk_scrolled_window_new(NULL, NULL);
   
  gtk_scrolled_window_set_policy((GtkScrolledWindow *) scrollwindow, 
				 GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC);
    
  clist1 = create_clist();
  RESCAN(); /* Makes initial call to create table of processes */

  gtk_scrolled_window_add_with_viewport((GtkScrolledWindow *) scrollwindow, 
					clist1);
  gtk_signal_connect(GTK_OBJECT(clist1),
                        "select_row",
                        select_row_callback,
                        NULL);

  gtk_widget_show(scrollwindow);

  gtk_box_pack_start(GTK_BOX (vbox1), scrollwindow, TRUE, TRUE, 0);
  
  separator = gtk_hseparator_new ();
  gtk_box_pack_start(GTK_BOX(vbox1), separator, FALSE, FALSE, 0);
  gtk_widget_show (separator);

  /* bottom buttons */
  hbox1 = gtk_hbox_new(FALSE,0);
  gtk_container_border_width(GTK_CONTAINER(hbox1), 3);

  button = gtk_button_new_with_label("Kill");
  gtk_signal_connect(GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (end_proc),
			     GTK_OBJECT (clist1));
  gtk_box_pack_start(GTK_BOX (hbox1), button, TRUE, TRUE, 0);
  gtk_widget_show(button);

  button = gtk_button_new_with_label("Terminate(-9)");
  gtk_signal_connect(GTK_OBJECT (button), "clicked",
		     GTK_SIGNAL_FUNC (kill_proc), GTK_OBJECT(clist1));

  gtk_box_pack_start(GTK_BOX (hbox1), button, TRUE, TRUE, 0);
  gtk_widget_show(button);
  
  button = gtk_button_new_with_label("Quit");
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_main_quit),
			     GTK_OBJECT (window));

  gtk_box_pack_start(GTK_BOX (hbox1), button, TRUE, TRUE, 0);
  gtk_widget_show(button);
  
  /* final packing */
  gtk_box_pack_start(GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
  gtk_widget_show(hbox1);

  gtk_container_add(GTK_CONTAINER (window), vbox1);

  gtk_widget_show (vbox1);
  gtk_widget_show  (window);
  gtk_main ();
  
  return(0);
}






















