/*
** Copyright (C) 6 July 1999 George Saj <gsaj@acm.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* This file handles piping to ps (for a given platform eventually 
   established with a configure script) */

/* some of these may be necessary :-) */
#include <stdio.h>
#include <stdlib.h>
#include <string.h> 
#include <unistd.h>
#include <gtk/gtk.h>
/* #include <errno.h> */

#define NEWLINE "\n"
#define SPACE " "
#define LINELENGTH 80
#define NUMCOL 11
#define PSCMD "ps auxw"

enum ps_col_linux {User, Pid, Cpu, Mem, Vsz, Rss, Tty, Stat, 
		   Start, Time, Command };

/* Static variables *********************/
static GtkWidget *clist1;
/**/

gboolean showed_columns[11] = {1, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1};

gboolean *get_showed_columns() {

     return showed_columns;
}

/* modified version of the glib function by a similar name
   The glib version doesn't handle two or more delimiters
   in a row. This removes "leading delimiters" on each token
   It's meant for single character delimiters. . .
   and also has a hack in it to handle where linux 2.0 has dates like 
   "Jul 20" on processes that have been running for some time.
   I'm not sure if that hack works yet. . .
 */
gchar** g_strsplit_gs (const gchar *string,
                       const gchar *delimiter,
                       const gint  max_tokens)
{
  GSList *string_list = NULL, *slist;
  gchar **str_array, *s;
  guint i, n = 1;
  gint num_tokens;
  g_return_val_if_fail (string != NULL, NULL);
  g_return_val_if_fail (delimiter != NULL, NULL);
  if (max_tokens < 1)
       num_tokens = G_MAXINT;
  s = strstr (string, delimiter);
  while(s && delimiter[0] == s[1])
       s++;
  if (s)
    {
      guint delimiter_len = strlen (delimiter);
      do
        {
          guint len;
          gchar *new_string;
          if (n == max_tokens) /* keeps last token intact */
               len = strlen (string);
          else
               len = s - string;
          new_string = g_new (gchar, len + 1);
          strncpy (new_string, string, len);
          new_string[len] = 0;
          string_list = g_slist_prepend (string_list, new_string);
          n++;
          string = s + delimiter_len;
          s = strstr (string, delimiter);
#ifdef OS_LINUX /* for linux 2.0 issues with day-old+ procs. . . */
	  if ((n - 2) == Time) {
	       gchar testtime[3] = s;
	       if ((strcasecmp(testtime, "jan") == 0 ||
		    strcasecmp(testtime, "feb") == 0 ||
		    strcasecmp(testtime, "mar") == 0 ||
		    strcasecmp(testtime, "apr") == 0 ||
		    strcasecmp(testtime, "may") == 0 ||
		    strcasecmp(testtime, "jun") == 0 ||
		    strcasecmp(testtime, "jul") == 0 ||
		    strcasecmp(testtime, "aug") == 0 ||
		    strcasecmp(testtime, "sep") == 0 ||
		    strcasecmp(testtime, "oct") == 0 ||
		    strcasecmp(testtime, "nov") == 0 ||
		    strcasecmp(testtime, "dec") == 0)) { 
		    printf ("%s|", s[3]);
		    s = s + 3;
	       }
#endif
          while(s && delimiter[0] == s[1])
               s++;
        } while (--num_tokens && s);
    }
  if (*string)
    {
      n++;
      string_list = g_slist_prepend (string_list, g_strdup (string));
    }
  str_array = g_new (gchar*, n);
  i = n -1;
  str_array[i--] = NULL;
  for (slist = string_list; slist; slist = slist->next)
    str_array[i--] = slist->data;
  g_slist_free (string_list);
  return str_array;
  }


void SetDefaultColumnWidth(GtkCList *clist1){
     int i;
     for (i = 0; i < calc_columns(); i++)
	  gtk_clist_set_column_auto_resize(clist1,i,1);
}


gint calc_columns(){
     gint i, num_columns = 0;
     for (i = 0; i < NUMCOL; i++)
	  if (showed_columns[i] == 1)
	       num_columns++;
     return num_columns;
}


GtkWidget *create_clist(){

     clist1 = gtk_clist_new(calc_columns());
     gtk_clist_column_titles_passive(GTK_CLIST(clist1));
     SetDefaultColumnWidth(GTK_CLIST(clist1));
     gtk_widget_show(clist1);
     return clist1;
}


void populate_clist(char *ps_lines){
     
     gchar **all_lines, **tokens;
     gchar *inserted_tokens[calc_columns()];
     gchar *word;
     gint row = 0, col = 0, last_col = 0;
     

     /* clear the clist */
     gtk_widget_hide(clist1);
     gtk_clist_clear (GTK_CLIST(clist1));

     /* first split up the lines */
     all_lines = g_strsplit(ps_lines, NEWLINE, 0); /* 0 for no max */
     /* first line contains column headings */
     tokens = g_strsplit_gs(all_lines[row++], SPACE, NUMCOL);
     for (col = 0; col < NUMCOL; col++) {
	  if (showed_columns[col] == 1) {
	       g_strdown(tokens[col]);  
	       gtk_clist_set_column_title (GTK_CLIST(clist1), 
					   last_col++, tokens[col]);
	  }
     }
     gtk_clist_column_titles_show(GTK_CLIST(clist1));
     /* now handle the rest of the tokens */
     while (all_lines[row] != NULL) {
	  last_col = 0;
	  tokens = g_strsplit_gs(all_lines[row], SPACE, NUMCOL);
	  for (col = 0; col < NUMCOL; col++) 
	       if (showed_columns[col] == 1) 
		    inserted_tokens[last_col++] = tokens[col];
	  

	  /* tests for gtaskman's PID and doesn't let it into the clist */
	  if (!((gint) g_strtod(tokens[1],0) == getpid())
	       && (strcmp(PSCMD, tokens[10])))
	       gtk_clist_append(GTK_CLIST(clist1),inserted_tokens);

	  g_strfreev(tokens);	  
	  row++;     
     }

     /* after done with the arrays, free hem using g_strfreev() */   
     g_strfreev(all_lines);
     gtk_widget_show(clist1);
}



void rescan_linux() {
  /* pipe fun to start */
  FILE *read_ps;
  char buffer[BUFSIZ + 1];
  int chars_read;

  char *line;

  memset(buffer, '\0',sizeof(buffer));
  read_ps = popen(PSCMD, "r"); /* w option - because wider is better :-) */

  if (read_ps != NULL) {
       chars_read = fread(buffer, sizeof(char), BUFSIZ, read_ps);
       pclose(read_ps);
    
       populate_clist(buffer);
  }
  else
       g_print("Error - ps produced no output!\nIf your ps works, report this to the maintainer <gsaj@acm.org>");
}   


















